<div ng-if="vm.projectFolderId" class="project-drive-folder">
  <website-iframe title="Project Drive Folder"
                  id="project-drive-folder"
-                 description="Resources for the project should be stored here. Open the folder in a new window and add it to your drive if you need to upload anything new!"
                  iframe-url="{{vm.projectFolderEmbedUrl}}"
                  external-url="{{vm.projectFolderExternalUrl}}">
  </website-iframe>
</div>
