# Generated by Django 2.2.13 on 2020-09-21 17:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0089_todolisttemplateimportrecord'),
    ]

    operations = [
        migrations.AddField(
            model_name='step',
            name='completion_ends_project',
            field=models.BooleanField(default=False),
        ),
    ]
