@import 'orchestra/static/orchestra/common/css/common';
@import 'orchestra/static/orchestra/common/css/base';

/* Login/account-related pages */
.login-wrap {
  font-size: 1.3rem;
  .error-non-field {
    padding: 0px 0px 10px 10px;
    color: red;
  }
  .error-field {
    padding: 5px 0px 0px 10px;
    color: red;
  }
}
