# Generated by Django 2.2.13 on 2021-02-11 21:57

from django.db import migrations, models
import orchestra.models.core.models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0092_autostaff_updates'),
    ]

    operations = [
        migrations.AlterField(
            model_name='todo',
            name='status',
            field=models.IntegerField(choices=[(0, 'pending'), (1, 'completed'), (2, 'declined')], default=0),
        ),
    ]

