# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-12 23:06
from __future__ import unicode_literals

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0033_alter_worker_and_assignment_on_timeentry'),
    ]

    operations = [
        migrations.AddField(
            model_name='timeentry',
            name='is_deleted',
            field=models.BooleanField(default=False),
        ),
    ]
