# Generated by Django 2.2.13 on 2021-04-19 17:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0093_todo_status_default'),
    ]

    operations = [
        migrations.RenameField(
            model_name='project',
            old_name='team_messages_url',
            new_name='scratchpad_url',
        ),
    ]
