# Generated by Django 2.2.13 on 2021-06-01 10:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0094_rename_team_messages_url'),
    ]

    operations = [
        migrations.AddField(
            model_name='todo',
            name='slug',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
