# Generated by Django 2.2.13 on 2020-09-15 14:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0087_todo_task_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='todo',
            name='task',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='todos_deprecated', to='orchestra.Task'),
        ),
    ]
