# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-10-02 18:47
from __future__ import unicode_literals

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0081_step_todolist_templates_to_apply'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='tags',
            field=jsonfield.fields.JSONField(default={'tags': []}),
        ),
    ]
