# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-07-10 21:18
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import orchestra.models.core.mixins
import orchestra.utils.models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0077_auto_20180629_1827'),
    ]

    operations = [
        migrations.CreateModel(
            name='TodoQA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('is_deleted', models.BooleanField(default=False)),
                ('comment', models.TextField(blank=True, null=True)),
                ('approved', models.NullBooleanField()),
                ('todo', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='qa', to='orchestra.Todo')),
            ],
            bases=(orchestra.models.core.mixins.TodoQAMixin, orchestra.utils.models.DeleteMixin, models.Model),
        ),
    ]
