.account-settings {

  .alert {
    margin-top: -70px;
  }

  a:hover, a:focus, a:visited {
    color: #FFF;
    text-decoration: none;
  }

  .row:not(:last-child) {
    margin-bottom: 15px;
  }

  .account-tabs {
    position: relative;
    z-index: 3;
    .btn {
      border-bottom-left-radius: 0;
      border-bottom-right-radius: 0;
      &.selected, &:hover{
        color: darken(#428bca, 15%);
        background-color: white;
        border-color: white;
      }
    }
  }

  .communication-methods {
    ul {
      padding: 5px 0px 0px 0px;
    }
    ul p label {
      font-size: 13px;
      font-weight: normal;
      letter-spacing: normal;
      line-height: inherit;
    }
  }

  .card {
    position: relative;
    z-index: 2;
    padding: 50px;
    background: #FFFFFF;
    box-shadow: 0px 17px 25px 0px rgba(104, 104, 104, 0.04), 0px 0px 55px 0px rgba(219, 219, 219, 0.28);
    height: auto;
  }

  .account-actions {
    margin-top: 15px;
    text-align: right;
  }

  label {
    font-size: 14px;
    font-weight: 600;
    letter-spacing: 1.59px;
    line-height: 26px;
  }

  input[type=checkbox] {
    height: inherit;
    width: auto;
    display: inline-block;
    margin-left: 3px;
    margin-right: 5px;
  }

  #id_phone_0, #id_phone_1 {
    display: inline;
  }

  #id_phone_0 {
    width: 29%;
    margin-right: 1%;
  }

  #id_phone_1 {
    width: 70%;
  }

  .availability-forms {
    display: flex;
    justify-content: space-between;
  }

  .availability-form {
    input {
      max-width: 100px;
    }

    td:first-child {
      text-align: right;
      padding: 0 12px 6px 0;
    }

    label {
      margin-bottom: 0;
      font-size: 13px;
      font-weight: 400;
    }

    &__heading {
      font-size: 14px;
      font-weight: 600;
    }
  }
}
