# Generated by Django 2.2.13 on 2020-08-18 16:28

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0085_rename_todo_description_to_title'),
    ]

    operations = [
        migrations.AddField(
            model_name='todo',
            name='additional_data',
            field=jsonfield.fields.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='todo',
            name='details',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='todo',
            name='order',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='todo',
            name='project',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='todos', to='orchestra.Project'),
        ),
        migrations.AddField(
            model_name='todo',
            name='section',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='todo',
            name='status',
            field=models.IntegerField(blank=True, choices=[(0, 'pending'), (1, 'completed'), (2, 'declined')], null=True),
        ),
        migrations.AddField(
            model_name='todo',
            name='step',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='todos', to='orchestra.Step'),
        ),
        migrations.AlterField(
            model_name='todo',
            name='task',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='todos', to='orchestra.Task'),
        ),
    ]
