# Generated by Django 2.2.13 on 2020-08-10 15:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0084_workflowversion_abort_completion_function'),
    ]

    operations = [
        migrations.RenameField(
            model_name='todo',
            old_name='description',
            new_name='title',
        ),
    ]
