.communication-container {
  text-align: center;

  .card {
    padding-top: 80px;
  }

  a {
    color: #FFF;
  }

  .spaced {
    padding-top: 15px;
  }
}


.staffing-container {
  font-size: 1.2em;
  .card {
    height: initial;

    h2 {
      padding: 125px 0px 125px 0px;
      text-align: center;
    }

    table {
      table-layout: fixed;

      td:first-child, th:first-child {
        padding-left: 20px;
      }

      .row-project {
        width: 15%;
      }
      .row-project-description {
        width: 20%;
      }
      .row-task {
        width: 20%;
      }
      .row-details {
        width: 20%;
      }
      .row-created-at {
        width: 10%;
      }

      .row-actions {
        width: 15%;
      }
    }
  }
}

.tags-col {
  max-width: 150px;
}

.tags-col span {
  display: inline-block;
  margin-right: 3px;
}
