# Generated by Django 2.2.13 on 2021-07-26 09:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0095_todo_slug'),
    ]

    operations = [
        migrations.AddField(
            model_name='todo',
            name='required',
            field=models.BooleanField(default=False),
        ),
    ]
